/****************************************************************************
 * Copyright (c) 2010-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
package com.ebmwebsourcing.easycommons.uuid;

import junit.framework.Assert;
import junit.framework.TestCase;

import com.ebmwebsourcing.easycommons.uuid.framework.orchestrator.GenUUIDFromMultipleQualifiedGeneratorOrchestrator;
import com.ebmwebsourcing.easycommons.uuid.framework.orchestrator.GenUUIDFromSingleQualifiedGeneratorOrchestrator;

public class QualifiedUUIDGeneratorTest extends TestCase {

    /**
     * Number of simultaneously launched uuid generator thread
     */
    private final int POOL_SIZE = 100;

    /**
     * Number of uuids generated by each thread
     */
    private final int ITERATIONS = 100;

    public void testGetNewID() {
        String domain = "foo";
        QualifiedUUIDGenerator generator = new QualifiedUUIDGenerator(domain);
        String uuid = generator.getNewID();
        String uuidPatern = "^" + domain
                + ":[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}$";
        assertTrue("Generated uuid must match the followwing pattern "
                + "'domain:[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}'",
                uuid.matches(uuidPatern));
    }

    public void testGetUuidMaxLength() {
        String domain = "foo";
        QualifiedUUIDGenerator generator = new QualifiedUUIDGenerator(domain);
        int uuidLength = generator.getUuidMaxLength();
        String uuid = generator.getNewID();
        assertEquals(uuid.length(), uuidLength);
    }

    public void testGenerateUniqueID() {
        try {
            System.gc();
            new GenUUIDFromMultipleQualifiedGeneratorOrchestrator().orchestrate(POOL_SIZE,
                    ITERATIONS);
        } catch (Exception e) {
            Assert.fail(e.getMessage());
        }
    }

    public void testGetNewIDMethodIsThreadsafe() {
        try {
            System.gc();
            new GenUUIDFromSingleQualifiedGeneratorOrchestrator()
                    .orchestrate(POOL_SIZE, ITERATIONS);
        } catch (Exception e) {
            Assert.fail(e.getMessage());
        }
    }

}
