/****************************************************************************
 * Copyright (c) 2010-2012, EBM WebSourcing - All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the University of California, Berkeley nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE REGENTS AND CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/
 
package com.ebmwebsourcing.easycommons.uuid.framework.orchestrator;

import java.util.Set;
import java.util.TreeSet;

import com.ebmwebsourcing.easycommons.thread.SimultaneousTaskExecutor;

public abstract class AbstractOrchestrator {

    private SimultaneousTaskExecutor simultaneousThreadExecutor = new SimultaneousTaskExecutor();

    protected abstract Runnable getGeneratorTask(final int nbIterates, final Set<String> uuids/*,
            final TestState testState*/);

    public class TestState {
        private boolean success = true;

        public boolean isSuccess() {
            return success;
        }

        public void setSuccess(boolean success) {
            this.success = success;
        }
    }

    public final void orchestrate(final int poolSize, final int iterations) throws Exception {

        // A set of generated uuids, used to test if no doublon is generated
        final Set<String> uuids = new TreeSet<String>();

        for (int i = 0; i < poolSize; i++) {
            Runnable task = this.getGeneratorTask(iterations, uuids);
            simultaneousThreadExecutor.registerTask(task);
        }
        
        simultaneousThreadExecutor.executeAllRegisteredTasks();
        
        System.out.println(uuids.size());
    }

}
